import { Link } from 'react-router-dom'
import { loadSavedChecklists } from '../hooks/useLocalStorage'
import { useState } from 'react'

interface SavedChecklistsProps {
  className?: string
}

export default function SavedChecklists({ className = '' }: SavedChecklistsProps) {
  const [savedChecklists, setSavedChecklists] = useState<{ uuid: string; name: string; lastOpened?: string }[]>(loadSavedChecklists())
  const handleForgetCheckList = (uuid: string): void => {
    const updatedChecklists = savedChecklists.filter((checklist: { uuid: string }) => checklist.uuid !== uuid)
    localStorage.setItem('gocheck-saved-checklists', JSON.stringify(updatedChecklists))
    setSavedChecklists(updatedChecklists)
  }


  if (savedChecklists.length === 0) {
    return (
      <div className={`bg-white dark:bg-gray-800 rounded-lg shadow-sm border border-gray-200 dark:border-gray-700 p-12 text-center ${className}`}>
        <div className="text-gray-400 dark:text-gray-500 text-6xl mb-4">📋</div>
        <h3 className="text-xl font-semibold text-gray-900 dark:text-white mb-2">No checklists yet</h3>
        <p className="text-gray-600 dark:text-gray-300">Create your first checklist above to get started!</p>
      </div>
    )
  }

  return (
    <div className={`bg-white dark:bg-gray-800 rounded-lg shadow-sm border border-gray-200 dark:border-gray-700 p-8 ${className}`}>
      <h2 className="text-2xl font-bold text-gray-900 dark:text-white mb-6">Your Checklists</h2>
      <div className="space-y-4">
        {savedChecklists.map((checklist) => (
          <div key={checklist.uuid} className="flex flex-col sm:flex-row items-start sm:items-center justify-between p-4 border border-gray-200 dark:border-gray-700 rounded-lg hover:bg-gray-50 dark:hover:bg-gray-700 transition-colors duration-200">
            <div className="flex-1 mb-4 sm:mb-0">
              <h3 className="text-lg font-semibold text-gray-900 dark:text-white mb-1">{checklist.name}</h3>
              <p className="text-sm text-gray-500 dark:text-gray-400">
                Last opened: {checklist.lastOpened ? new Date(checklist.lastOpened).toLocaleString() : 'Never'}
              </p>
            </div>
            <div className="flex items-center gap-3">
                <Link to={`/${checklist.uuid}`}>
              <button 
                className="px-4 py-2 bg-blue-600 dark:bg-blue-500 text-white font-medium rounded-lg hover:bg-blue-700 dark:hover:bg-blue-600 focus:outline-none focus:ring-2 focus:ring-blue-500 focus:ring-offset-2 dark:focus:ring-offset-gray-800 transition-colors duration-200"
              >
                Open
              </button>
                </Link>
              <button 
                onClick={() => {
                  const url = `${window.location.origin}/${checklist.uuid}`
                  navigator.clipboard.writeText(url).then(() => {
                    // Optional: Add a toast notification here
                    alert('Checklist link copied to clipboard! ' + url)
                  }).catch(() => {
                    // Fallback for older browsers
                    const textArea = document.createElement('textarea')
                    textArea.value = url
                    document.body.appendChild(textArea)
                    textArea.select()
                    document.execCommand('copy')
                    document.body.removeChild(textArea)
                    alert('Checklist link copied to clipboard!')
                  })
                }}
                className="px-4 py-2 bg-green-600 dark:bg-green-500 text-white font-medium rounded-lg hover:bg-green-700 dark:hover:bg-green-600 focus:outline-none focus:ring-2 focus:ring-green-500 focus:ring-offset-2 dark:focus:ring-offset-gray-800 transition-colors duration-200"
                title="Share checklist link"
              >
                📤 Share
              </button>
              <button 
                onClick={() => handleForgetCheckList(checklist.uuid)}
                className="px-4 py-2 bg-red-600 dark:bg-red-500 text-white font-medium rounded-lg hover:bg-red-700 dark:hover:bg-red-600 focus:outline-none focus:ring-2 focus:ring-red-500 focus:ring-offset-2 dark:focus:ring-offset-gray-800 transition-colors duration-200"
              >
                Forget
              </button>
            </div>
          </div>
        ))}
      </div>
    </div>
  )
} 