import { useEffect, useRef, useState } from 'react'
import type { SSEEvent, ChecklistItem } from '../types'
import { saveChecklist } from '../hooks/useLocalStorage'

export function useSSE(uuid: string) {
  const [items, setItems] = useState<ChecklistItem[]>([])
  const [checkListName, setCheckListName] = useState('')
  const [isConnected, setIsConnected] = useState(false)
  const [error, setError] = useState<string | null>(null)
  const eventSourceRef = useRef<EventSource | null>(null)

  useEffect(() => {
    if (!uuid) return

    const connectSSE = () => {
      try {
        const eventSource = new EventSource(`/api/checklists/${uuid}/sse`)
        eventSourceRef.current = eventSource

        eventSource.onopen = () => {
          setIsConnected(true)
          setError(null)
        }

        eventSource.onmessage = (event) => {
          try {
            const data: SSEEvent = JSON.parse(event.data)
            
            switch (data.type) {
              case 'checklist_name':
                if (data.name) {
                  saveChecklist({
                    uuid,
                    name: data.name,
                    createdAt: new Date().toISOString(),
                  })
                  setCheckListName(data.name)
                }
                break
              case 'full_state':
                if (data.items) {
                  setItems(data.items)
                }
                break
              case 'item_added':
                if (data.item) {
                  setItems(prev => [...prev, data.item!])
                }
                break
              case 'item_updated':
                if (data.item) {
                  setItems(prev => 
                    prev.map(item => 
                      item.id === data.item!.id ? data.item! : item
                    )
                  )
                }
                break
              case 'item_deleted':
                if (data.id) {
                  setItems(prev => prev.filter(item => item.id !== data.id))
                }
                break
              case 'item_locked':
                if (data.id && data.locked_by && data.expires) {
                  setItems(prev => 
                    prev.map(item => 
                      item.id === data.id 
                        ? { 
                            ...item, 
                            locked_by: data.locked_by, 
                            lock_until: data.expires 
                          }
                        : item
                    )
                  )
                }
                break
              case 'item_unlocked':
                if (data.id) {
                  setItems(prev => 
                    prev.map(item => 
                      item.id === data.id 
                        ? { ...item, locked_by: undefined, lock_until: undefined }
                        : item
                    )
                  )
                }
                break
            }
          } catch (parseError) {
            console.error('Error parsing SSE message:', parseError)
          }
        }

        eventSource.onerror = (error) => {
          console.error('SSE Error:', error)
          setError('Connection lost. Trying to reconnect...')
          setIsConnected(false)
          
          // Close and attempt to reconnect after a delay
          eventSource.close()
          setTimeout(connectSSE, 3000)
        }

      } catch (error) {
        console.error('Error connecting to SSE:', error)
        setError('Failed to connect to server')
        setIsConnected(false)
      }
    }

    connectSSE()

    return () => {
      if (eventSourceRef.current) {
        eventSourceRef.current.close()
        eventSourceRef.current = null
      }
      setIsConnected(false)
    }
  }, [uuid])

  return { items, checkListName, isConnected, error }
} 